//
//  FrameMonitor.h
//
#ifndef FrameMonitor_Defs
#define FrameMonitor_Defs

#include <time.h>
#include <vector>
#include <deque>
#include <map>
#include <string>

#include "main_classes.h"
#include "workfield_class.h"
#include "WFTabs.h"

#include "sysmac.h"
#include "window_init_data.h"
#include "DBufClass.h"

#include "console_class.h"
#include "FastDelegate.h"
#include "controls.h"
#include "status_bar.h"

#include "FrameStream.h"


#pragma warning(disable:4996)


#define  FrameMonitor_Interface_CLASS_NAME  "FrameMonitor_Interface"

const dword  FrameMonitor_Interface_Generation = 1;
const dword  FrameMonitor_Interface_Version    = 1;

class FrameMonitor_Interface : public InterfaceClass {
public:
	static const char* const CLASS_NAME; // "FrameMonitor_Interface"

	virtual void ShowFrame( BGR_Frame_Type *pFrame )=0;

protected:
	virtual ~FrameMonitor_Interface(){}
	FrameMonitor_Interface( ObjectClass *pOwner ):InterfaceClass( pOwner, CLASS_NAME, FrameMonitor_Interface_Generation, FrameMonitor_Interface_Version ){}
};



class FrameMonitorClass : protected WorkfieldOwnerClass, protected FrameMonitor_Interface {
protected:
	virtual ~FrameMonitorClass();
	
	virtual void ShowFrame( BGR_Frame_Type *pFrame );

	void ResizeToRECT( RECT rc );
	
	HWND  GetFocusHWND() { return NULL; }

	BGR_Frame_Type  *pFrame;

	FrameMonitorClass( ConfigInterface *pConfig, const char *acAlias );
	
	HWND  hWndOuterFrame;
	HDC   hdcOuterFrame;
	
	IntegerParameter  FrameBackground;
	dword             dwFrameBackground;
	HBRUSH            hFrameBackground;
	void              FrameBackground_ParameterController( const char*, int nVal );
	
	HWND  hWndFrameMonitorMain;
	HWND  hWndFrameMonitorScreen;
	HDC   hdcFrameMonitorScreen;
	
	int  nFrameWidth;
	int  nFrameHeight;
	
	static const int nMainBorderWidth = 1;
	static const int nScreenBorderWidth = 2;

	void  CreateFrameMonitorMain();
	void  ResizeFrameMonitorMain();

	WorkfieldOwnerStatusBarClass  *pFrameMonitorStatus;

	static LRESULT CALLBACK WndProcT3( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam );
	LRESULT CALLBACK FrameWndProc( HWND, UINT, WPARAM, LPARAM );

	static LRESULT CALLBACK WndProcT1( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam );
	LRESULT CALLBACK MainWndProc( HWND, UINT, WPARAM, LPARAM );

	static LRESULT CALLBACK WndProcT2( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam );
	LRESULT CALLBACK ScreenWndProc( HWND, UINT, WPARAM, LPARAM );


private:
	bool bObjectDestroing;
	bool bWindowDestroed;

	static unsigned int nObjectsCount;
};

#endif