//
//  abstract_model.h  -     (       , 
//                           ,   ).
//

#ifndef abstract_modelDefs
#define abstract_modelDefs 1

#include <string>
#include <vector>
#include <map>
#include <deque>
#include <algorithm>
#include <functional>

#include "sysmac.h"
#include "FastDelegate.h"
#include "console_class.h"

#pragma warning(disable:4996)


const int BUS_WAIT_TIMEOUT = 15;

///////////////////////

class AbstractBusInterface;

struct TimedCallbackStruct {
	ObjectClass                   *pObject;
	fastdelegate::FastDelegate0<>  Callback;
	dword                          dwAwaitedTickCount;
	dword                          dwControlTickCount;
	dword                          dwInitialTickCount;
	dword                          uCallbacksPerSecond;
	qword                          qwCallbacksCount;
	AbstractBusInterface          *pSecondaryABusI;
};

struct NS_IntervalCallbackStruct {
	ObjectClass                   *pObject;
	fastdelegate::FastDelegate0<>  Callback;
	dword                          dwIntervalNS;
	bool					       bPeriodical;
	qword                          qwAwaitedRunTimeNS;
	qword                          qwInitialRunTimeNS;
	qword                          qwCallbacksCount;
};


typedef std::vector<ObjectClass*> ObjectPointersVectorType;


#define  AbstractBusInterface_CLASS_NAME   "AbstractBusInterface"

const dword  AbstractBusInterface_Generation = 1;
const dword  AbstractBusInterface_Version    = 1;


class AbstractBusInterface : public InterfaceClass {
public:
	static const char* const CLASS_NAME; // "AbstractBusInterface"

	virtual ObjectClass*  Get_pCPU()=0;
	virtual void          SetHostEvent()=0;
	virtual void          CPU_SpeedChanged()=0;
	
	virtual qword  Get_qwClocksCount()=0;
	virtual qword  Get_qwStepCount()=0;
	virtual qword  Get_qwRunTimeNS()=0;
	virtual dword  Get_dwFclkKHz()=0;
	virtual void   Shorten_Remaining_BurstSteps_to( int nSteps )=0;
	
	virtual const ObjectPointersVectorType&  GetInstalledDevices()=0;
	virtual const ObjectPointersVectorType&  GetLinkedTools()=0;
	virtual const ObjectPointersVectorType&  GetSecondaryBuses()=0;
	
	virtual void SetTimedCallback(        ObjectClass *pObject, fastdelegate::FastDelegate0<> Callback, dword dwAwaitedTickCount,  AbstractBusInterface *pSecondaryABusI = NULL )=0;
	virtual void SetPeriodicalCallback(   ObjectClass *pObject, fastdelegate::FastDelegate0<> Callback, dword uCallbacksPerSecond, AbstractBusInterface *pSecondaryABusI = NULL )=0;
	virtual void Set_NS_IntervalCallback( ObjectClass *pObject, fastdelegate::FastDelegate0<> Callback, dword dwIntervalNS, bool bPeriodical = false )=0;
	virtual void Clear_NS_IntervalCallback( fastdelegate::FastDelegate0<> Callback )=0;
	virtual void Make_TCS_Callback( TimedCallbackStruct *pTCS )=0;
	
	virtual void SetPrimaryBus( ObjectClass *pPrimaryBus )=0;
	virtual void AddSecondaryBus( ObjectClass *pSecondaryBus )=0;

protected:
	virtual ~AbstractBusInterface(){}
	AbstractBusInterface( ObjectClass *pOwner ):InterfaceClass( pOwner, CLASS_NAME, AbstractBusInterface_Generation, AbstractBusInterface_Version ){}
};

/////
#define  AbstractDevice_Interface_CLASS_NAME  "AbstractDevice_Interface"

const dword  AbstractDevice_Interface_Generation = 1;
const dword  AbstractDevice_Interface_Version    = 1;

class AbstractDevice_Interface : public InterfaceClass {
public:
	static const char* const CLASS_NAME; // "AbstractDevice_Interface"

	virtual void PrepareEmulation()=0;

protected:
	virtual ~AbstractDevice_Interface(){}
	AbstractDevice_Interface( ObjectClass *pOwner ):InterfaceClass( pOwner, CLASS_NAME, AbstractDevice_Interface_Generation, AbstractDevice_Interface_Version ){}
};


/////
#define  AbstractCPU_Interface_CLASS_NAME  "AbstractCPU_Interface"

const dword  AbstractCPU_Interface_Generation = 1;
const dword  AbstractCPU_Interface_Version    = 1;

class AbstractCPU_Interface : public InterfaceClass {
public:
	static const char* const CLASS_NAME; // "AbstractCPU_Interface"

	virtual qword  Get_qwClocksCount(){ return 0; }
	virtual qword  Get_qwStepCount()  { return 0; }
	virtual qword  Get_qwRunTimeNS()=0;
	virtual dword  Get_dwFclkKHz()    { return 0; }
	virtual dword  Get_dwAvgStepLatencyNS()=0;
	virtual void   Shorten_Remaining_BurstSteps_to( int nSteps )=0;
	virtual void   StepsLogStrOut( const char *pStr ){}


	virtual bool   TrueSpeedSupported(){ return bTrueSpeedSupported; }

	virtual void   SetBurstSize(     int nBurstSIZE     )=0;
	virtual void   SetTempBurstSize( int nTempBurstSIZE )=0;
	virtual int    GetBurstSize()=0;

protected:
	virtual ~AbstractCPU_Interface(){}
	AbstractCPU_Interface( bool b_TrueSpeedSupported,  ObjectClass *pOwner ):
	InterfaceClass( pOwner, CLASS_NAME, AbstractCPU_Interface_Generation, AbstractCPU_Interface_Version ),
	bTrueSpeedSupported(b_TrueSpeedSupported)
	{}
	
	bool bTrueSpeedSupported;
};

/////

class AbstractBusClass : protected BusClass, protected AbstractBusInterface {
protected:
	virtual ~AbstractBusClass(){ if(hHostEvent) CloseHandle(hHostEvent); }
	AbstractBusClass( const char *acArchitecture );

	BusInterface                            *pPrimaryBusI;
	AbstractBusInterface                    *pPrimaryABusI;
	virtual void SetPrimaryBus( ObjectClass *pPrimaryBus );
	
	ObjectClass                 *pCPU;
	AbstractCPU_Interface       *pCPU_I;

	virtual ObjectClass*   Get_pCPU() { return pCPU; }

	fastdelegate::FastDelegate0<>    CPU_Callback;
	
	ObjectPointersVectorType         vpInstalledDevices;
	ObjectPointersVectorType         vpLinkedTools;
	ObjectPointersVectorType         vpSecondaryBuses;

	virtual void AddSecondaryBus( ObjectClass *pSecondaryBus );

	virtual const ObjectPointersVectorType&   GetInstalledDevices() { return vpInstalledDevices; }
	virtual const ObjectPointersVectorType&   GetLinkedTools()      { return vpLinkedTools;      }
	virtual const ObjectPointersVectorType&   GetSecondaryBuses()   { return vpSecondaryBuses;   }
	
	virtual bool PrepareEmulation();
	
	void PrepareAbstractDevice( ObjectClass *pObject );
	
	virtual void   Make_TCS_Callback( TimedCallbackStruct *pTCS );
	virtual void   SetTimedCallback(        ObjectClass *pObject, fastdelegate::FastDelegate0<> Callback, dword dwAwaitedTickCount,  AbstractBusInterface *pSecondaryABusI );
	virtual void   SetPeriodicalCallback(   ObjectClass *pObject, fastdelegate::FastDelegate0<> Callback, dword uCallbacksPerSecond, AbstractBusInterface *pSecondaryABusI );
	virtual void   Set_NS_IntervalCallback( ObjectClass *pObject, fastdelegate::FastDelegate0<> Callback, dword dwIntervalNS, bool bPeriodical );
	virtual void   Clear_NS_IntervalCallback( fastdelegate::FastDelegate0<> Callback );
	virtual bool   IsEmulThreadActive() 
	{
		if( pPrimaryBusI ) { return pPrimaryBusI->IsEmulThreadActive(); }
		return bEmulThreadActive;
	}
	virtual void   StopEmulation();
	virtual void   CPU_SpeedChanged()   { bCPU_SpeedChanged = true; }
	virtual void   SetHostEvent()       { SetEvent( hHostEvent );  }
	virtual dword  EmulThread();
	
	virtual qword  Get_qwClocksCount()
	{
		if( !pCPU_I ) { return 0; }
		return pCPU_I->Get_qwClocksCount();
	}
	virtual qword  Get_qwStepCount()
	{
		if( !pCPU_I ) { return 0; }
		return pCPU_I->Get_qwStepCount();
	}
	virtual qword  Get_qwRunTimeNS()
	{
		if( !pCPU_I ) { return 0; }
		return pCPU_I->Get_qwRunTimeNS();
	}
	virtual dword  Get_dwFclkKHz()
	{
		if( !pCPU_I ) { return 0; }
		return pCPU_I->Get_dwFclkKHz();
	}
	virtual void   Shorten_Remaining_BurstSteps_to( int nSteps )
	{
		if( !pCPU_I ) { return; }
		return pCPU_I->Shorten_Remaining_BurstSteps_to( nSteps );
	}
	
//
	safe_deque< TimedCallbackStruct >        TCQ;
	safe_deque< NS_IntervalCallbackStruct >  ICQ;

	bool bEmulThreadActive;
	bool bStopEmulation;

	HANDLE  hHostEvent;

	void  Pause_StateCallback( const char*, int );
	void  Power_StateCallback( const char*, int );
	void  Reset_StateCallback( const char*, int );
	
	StateVarType  St_Power;
	StateVarType  St_Reset;


	int    nPeriodicalCallbacksPerSecond;
	dword  uCPU_CallbacksCount;
	bool   bCPU_SpeedChanged;
	
	void   TuneBurstSize();
	void   CheckIntervalCallbacks();

	std::string sPrepareEmulation_Header;
	
private:
	int  LinkCheck( SocketClass*, SocketClass* );
	void LinkDone(  SocketClass*, SocketClass* );
	
	int   nSleep;
	dword uLastCalibrateTC;
};

//////////////////////////////////////

class AbstractBusUserClass : protected BusUserClass {
protected:
	virtual ~AbstractBusUserClass(){}
	AbstractBusUserClass( const char *acArchitecture );

	AbstractBusInterface  *pABusI;
	
private:
	void BusReceiver( PVOID ){}

	int  LinkCheck( SocketClass*, SocketClass* );
	void LinkDone(  SocketClass*, SocketClass* );
};

//////////////////////////////////////

class AbstractDeviceClass : protected AbstractBusUserClass, protected AbstractDevice_Interface {
protected:
	virtual ~AbstractDeviceClass(){}
	AbstractDeviceClass( const char *acArchitecture );
	
	StateVarType  St_Power;
	StateVarType  St_Reset;
};


///////////////////////////////////////

#define  Abstract_DSK_DeviceInterface_CLASS_NAME   "Abstract_DSK_DeviceInterface"

const dword  Abstract_DSK_DeviceInterface_Generation = 1;
const dword  Abstract_DSK_DeviceInterface_Version    = 1;


class Abstract_DSK_DeviceInterface : public InterfaceClass {
public:
	static const char* const CLASS_NAME; // "Abstract_DSK_DeviceInterface"

	virtual int  ReadDSK(  int nUnit, int nBlock, int nBytes, void *pBuf )=0;
	virtual int  WriteDSK( int nUnit, int nBlock, int nBytes, void *pBuf )=0;
	
protected:
	virtual ~Abstract_DSK_DeviceInterface(){}
	Abstract_DSK_DeviceInterface( ObjectClass *pOwner ):InterfaceClass( pOwner, CLASS_NAME, Abstract_DSK_DeviceInterface_Generation, Abstract_DSK_DeviceInterface_Version ){}
};

/////

class Abstract_DSK_DeviceClass : protected StorageMediaUserInterface, virtual protected AbstractDeviceClass, protected Abstract_DSK_DeviceInterface {
protected:
	virtual ~Abstract_DSK_DeviceClass(){}
	Abstract_DSK_DeviceClass( const char *acArchitecture, const char *acDriveName, const int nUnitsTotal, const char *acDefExt ):
	ObjectClass( "Abstract_DSK_DeviceClass" ),
	StorageMediaUserInterface(    (ObjectClass*)this, acDriveName, nUnitsTotal, acDefExt ),
	Abstract_DSK_DeviceInterface( (ObjectClass*)this ),
	AbstractDeviceClass( acArchitecture )	
	{
	}
};


//////////////////////////////////////

class AbstractCPU_Class : virtual protected AbstractDeviceClass, protected AbstractCPU_Interface {
protected:
	virtual ~AbstractCPU_Class(){}
	AbstractCPU_Class( const char *acArchitecture, bool bTrueSpeedSupported );

	StateVarType   St_Pause;

	virtual void PrepareEmulation();

	virtual void CPU_Steps( dword nSteps )=0;

	int      nBurstSize;
	int      nDefaultBurstSize;
	
	virtual void   SetBurstSize( int nBurstSIZE )
	{
		if( nBurstSIZE ) 
		{
			nDefaultBurstSize = nBurstSIZE; 
			nBurstSize        = nBurstSIZE;
		}
	}
	virtual void   SetTempBurstSize( int nTempBurstSIZE )
	{
		if( nTempBurstSIZE ) { nBurstSize = nTempBurstSIZE;    }
		else                 { nBurstSize = nDefaultBurstSize; }
	}
	virtual int    GetBurstSize() 
	{
		if( uKIPS_Limit == 1 ) { return 1; }
		return nBurstSize;
	}
	
	static const NamedValType                       aValidCpuSpeeds[];
	static const std::vector< const NamedValType >  vValidCpuSpeeds;
	
	dword	uLast_KIPS_Limit; //  KIPS = MIPS*1000
	dword	uKIPS_Limit;
	dword   uInitialTC;

	IntegerParameter nP_KIPS_Limit;
	void KIPS_Limit_ParameterCallback( const char*, int nVal );
	
	void  Power_StateCallback( const char*, int nState );
	void  Reset_StateCallback( const char*, int nState );
	void  Pause_StateCallback( const char*, int nState );
	
	dword   nStepCount;
	qword	qwSteps;
	qword	qwStepsTotal;
	
	dword   dwCurrentStepsClocksCount;
	qword	qwClocksCount;
	
//	qword   qwCPU_PoweredRunTimeNS;
	qword   qwCPU_TotalRunTimeNS;

	dword   dwAvgStepLatencyNS;
	qword   qwTotalStepsLatencyNS;
	int     nLastLatencyDiff;
	dword   dwLastLatencyDiff_TC;
	dword   dwLastBadDiff_TC;
	int     nThreadPriority;
	
	IntegerConstant  nC_CPU_FrequencyKHZ;

	virtual dword  Get_dwFclkKHz()    { return nC_CPU_FrequencyKHZ;       }
	virtual qword  Get_qwStepCount()  { return qwStepsTotal + nStepCount; }
	virtual qword  Get_qwRunTimeNS()  
	{
		if( uKIPS_Limit == 1 )
		{
			return qwCPU_TotalRunTimeNS + ((qword)dwCurrentStepsClocksCount * 1000000)/nC_CPU_FrequencyKHZ;
		}
		else
		{
			return qwCPU_TotalRunTimeNS + nStepCount * dwAvgStepLatencyNS;
		}
	}

	virtual qword  Get_qwClocksCount(){ return qwClocksCount + dwCurrentStepsClocksCount; }
	virtual dword  Get_dwAvgStepLatencyNS(){ return dwAvgStepLatencyNS; }


private:
	void  TimedCallback();

	qword   qwCPU_ContinuousRunTimeNS;
};
//*/
//////////////////////////////////////

#endif