//
//    controls.h
//
#pragma warning(disable:4996)

#include <windows.h>
#include <stdio.h>
#include <string>
#include <vector>

#include "sysmac.h"
#include "main_classes.h"


#include <Uxtheme.h>
#pragma comment(lib, "UxTheme.lib")

//////////////////////////////////
typedef unsigned char        byte;
typedef unsigned short       word;
typedef unsigned long       dword;
typedef unsigned long long  qword;
//////////////////////////////////

#ifndef ControlsDef
#define ControlsDef 1

extern char acModuleName[];

////////////////////////////////////////////////////////////

const dword  ControlsClass_Generation = 1;
const dword  ControlsClass_Version    = 1;


class ControlsClass : public GlobalClass {
public:
	virtual int  Width()=0;
	virtual int  Height()=0;
	virtual void SetX0( int nX0 )=0;
	virtual void SetY0( int nY0 )=0;
	virtual void Show()=0;
	virtual void Hide()=0;
	virtual void RePaint()=0;
	virtual void SetDefaultBackground( COLORREF dwDefaultBackground ) {}
	virtual void DisableOnColors(){};
	virtual void EnableOnColors(){};
	virtual bool IsThemed(){ return false; }
	virtual void Destroy() { delete this; }
	
protected:
	ControlsClass():GlobalClass( ControlsClass_Generation, ControlsClass_Version ){}
	virtual ~ControlsClass(){}
	
	std::string sControlName;
};

////////////////////////////////////////////////////////////

#define FLASH_BUTTON_MSG   42010

#define DEF_OFF_BACKGROUND    RGB(0xC0,0xC0,0xC0)
#define DEF_ON_BACKGROUND     RGB(0xC0,0xC0,0xC0)
#define DEF_BACKGROUND        -1

enum PushModeType : int { PUSH_NONE, PUSH_NOFIX, PUSH_FIX };

struct FlashButtonInitData {
	dword        dwLparam;
	int          nFontDY;
	int          nFontWeight;
	char        *pON_Text;
	char        *pOFF_Text;
	PushModeType PushMode;
	bool         bInverted;
	int          nPushed_Text_X_Offset;
	int          nPushed_Text_Y_Offset;
	int          nPopped_Text_X_Offset;
	int          nPopped_Text_Y_Offset;
	int          nUserText_X_Offset;
	LONG         dwPushed_Style;
	LONG         dwPushed_ExStyle;
	LONG         dwPopped_Style;
	LONG         dwPopped_ExStyle;
	int          nWidth;
	int          nHeight;
	HWND         hWndParent;
	HWND         hWndReceiver;
	int          nX0;
	int          nY0;
	COLORREF     dwON_TextColor;
	COLORREF     dwOFF_TextColor;
	COLORREF     dwON_Background;
	COLORREF     dwOFF_Background;
	COLORREF     dwDefaultBackground;
	bool         bButtonLike_PoppedBorder;
	bool         bButtonLike_PushedBorder;
	bool         bTextXCenter;
	
	fastdelegate::FastDelegate1<int> Push_Delegate;
	
	FlashButtonInitData():
	dwLparam(0),
	nFontDY(17),
	nFontWeight(500),
	pON_Text(NULL),
	pOFF_Text(NULL),
	dwON_TextColor(-1),
	dwOFF_TextColor(-1),
	PushMode(PUSH_NONE),
	bInverted(false),
	nPushed_Text_X_Offset(0),
	nPushed_Text_Y_Offset(0),
	nPopped_Text_X_Offset(0),
	nPopped_Text_Y_Offset(0),
	nUserText_X_Offset(0),
	dwPushed_Style( WS_CHILD|WS_VISIBLE ),
	dwPushed_ExStyle(0),
	dwPopped_Style( WS_CHILD|WS_VISIBLE ),
	dwPopped_ExStyle(0),
	nWidth(10),
	nHeight(18),
	hWndParent(NULL),
	hWndReceiver(NULL),
	nX0(0),
	nY0(0),
	dwON_Background( DEF_BACKGROUND ),
	dwOFF_Background( DEF_BACKGROUND ),
	dwDefaultBackground( DEF_BACKGROUND ),
	bButtonLike_PoppedBorder(true),
	bButtonLike_PushedBorder(true),
	bTextXCenter(false)
	{}
};


class FlashButtonClass : public ControlsClass {
public:
	static const char* const CLASS_NAME; // = "FlashButton";
	
	FlashButtonClass( FlashButtonInitData* );

	virtual int  Width() { return nWidth; }
	virtual int  Height(){ return nHeight; }
	virtual void SetX0( int nX0 );
	virtual void SetY0( int Y0 )
	{
		nY0 = Y0;
		MoveWindow( hWndPush, nX0, nY0, nWidth, nHeight, true );
	}
	virtual void Show(){ ShowWindow( hWndPush, SW_SHOW ); bHidden = false; }
	virtual void Hide(){ ShowWindow( hWndPush, SW_HIDE ); bHidden = true; }
	virtual void RePaint()
	{
		bool bState = (bool)nPushState;
		if( bInverted ) { bState = !bState; }
		nPushState = -1;
		SetState( bState );
	}
	
	virtual void SetDefaultBackground( COLORREF dwBackground ) 
	{
		dwDefaultBackground = dwBackground;
		RePaint();
	}
	
	virtual void SetState( int nState, bool bTemporal = false );
	virtual void SetUserText( const char *pcText, int nUserText_X_Offset = 0 );
	virtual void ClearUserText(){ sUserText.clear(); nUserText_X_Offset = 0; }
	virtual void SetON_Text( const char *pcText )
	{
		sON_Text = pcText;
		if( nPushState )
		{ 
			sText = sON_Text;
			Paint();
		}
		Paint();
	}
	virtual void SetOFF_Text( const char *pcText )
	{
		sOFF_Text = pcText;
		if( nPushState == bInverted )
		{ 
			sText = sOFF_Text;
			Paint(); 
		}
	}
	
	virtual void SetBgColor( COLORREF dwBackgrownd );
//	virtual void SetTextColor( COLORREF dwColor );
	
	virtual void DisableButton();
	virtual void EnableButton();
	
	virtual void DisableOnColors();
	virtual void EnableOnColors();

protected:
	virtual ~FlashButtonClass();

	static LRESULT CALLBACK WndProc0( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam );
	LRESULT CALLBACK WndProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam );
	
	void  InitInternals( FlashButtonInitData *pInitData );
	
	void  Paint();

	void  Set_ON_BgColor()
	{
		bSystemBackground = false;
		
//		if( dwON_Background == DEF_BACKGROUND || bDisabled )
		if( dwON_Background == DEF_BACKGROUND )
		{
			if( dwDefaultBackground != DEF_BACKGROUND )
			{
				SetBgColor( dwDefaultBackground );
			}
			else
			{
				bSystemBackground = true;
				SetBgColor( GetSysColor( COLOR_3DFACE ) );
			}
		}
		else
		{
			SetBgColor( dwON_Background );
		}
	}

	void  Set_ON_TextColor()
	{
		if( !bDisabled )
		{
			if( dwON_TextColor == -1 )
			{
				if( dwON_Background == DEF_BACKGROUND )
				{
					::SetTextColor( hDCPush, GetSysColor( COLOR_BTNTEXT ) );
				}
				else
				{
					::SetTextColor( hDCPush, 0 );
				}
			}
			else
			{
				::SetTextColor( hDCPush, dwON_TextColor );
			}
		}
		else
		{
			::SetTextColor( hDCPush, RGB(0x70,0x70,0x70) );
		}
	}
	void  Set_OFF_BgColor()
	{
		bSystemBackground = false;
		
//		if( dwOFF_Background == DEF_BACKGROUND || bDisabled )
		if( dwOFF_Background == DEF_BACKGROUND )
		{
			if( dwDefaultBackground != DEF_BACKGROUND )
			{
				SetBgColor( dwDefaultBackground );
			}
			else
			{
				bSystemBackground = true;
				SetBgColor( GetSysColor( COLOR_3DFACE ) );
			}
		}
		else
		{
			SetBgColor( dwOFF_Background );
		}
	}
	void  Set_OFF_TextColor()
	{
		if( !bDisabled )
		{
			if( dwOFF_TextColor == -1 )
			{
				if( dwOFF_Background == DEF_BACKGROUND )
				{
					::SetTextColor( hDCPush, GetSysColor( COLOR_BTNTEXT ) );
				}
				else
				{
					::SetTextColor( hDCPush, 0 );
				}
			}
			else
			{
				::SetTextColor( hDCPush, dwOFF_TextColor );
			}
		}
		else
		{
			::SetTextColor( hDCPush, RGB(0x70,0x70,0x70) );
		}
	}
//
	int   nPushState;
	fastdelegate::FastDelegate1<int> Push_Delegate;

	HWND  hWndPush;
	HDC   hDCPush;
	
	HWND  hWndParent;
	HWND  hWndReceiver;

	dword        dwLparam;
	bool         bButtonLike_PoppedBorder;
	bool         bButtonLike_PushedBorder;
	COLORREF     dwON_Background;
	COLORREF     dwOFF_Background;
	COLORREF     dwDefaultBackground;
	COLORREF     dwON_TextColor;
	COLORREF     dwOFF_TextColor;
	COLORREF     dwUser_TextColor;
	int          nStyle;
	int          nX0;
	int          nY0;
	int          nWidth;
	int          nHeight;
	int          nPushed_Text_X_Offset;
	int          nPushed_Text_Y_Offset;
	int          nPopped_Text_X_Offset;
	int          nPopped_Text_Y_Offset;
	int          nUserText_X_Offset;
	LONG         dwPushed_Style;
	LONG         dwPushed_ExStyle;
	LONG         dwPopped_Style;
	LONG         dwPopped_ExStyle;
	PushModeType PushMode;
	bool         bInverted;
	
	bool         bTextXCenter;

	int          nText_X_Offset;
	int          nText_Y_Offset;
	
	
	HFONT fontX;
	int   nFontDY;
	int   nFontDX;

	HGDIOBJ  hBackground;
	
	std::string sON_Text;
	std::string sOFF_Text;
	std::string sUserText;
	std::string sText;
	
	dword  dwWindowThreadID;
	
	bool   bDisabled;
	bool   bHidden;
	bool   bUserTextColor;
	bool   bOnColorsEnabled;
	
	bool   bSystemBackground;
	
	bool   bObjectDestroing;
	bool   bWindowDestroed;
};


////////////////////////////////////////////////////////////

struct StateFlashButtonInitData {
	int            nFontDY;
	int            nFontWeight;
	char          *pON_Text;
	char          *pOFF_Text;
	PushModeType   PushMode;
	bool           bInverted;
	int            nPushed_Text_X_Offset;
	int            nPushed_Text_Y_Offset;
	int            nPopped_Text_X_Offset;
	int            nPopped_Text_Y_Offset;
	int            nUserText_X_Offset;
	LONG           dwPushed_Style;
	LONG           dwPushed_ExStyle;
	LONG           dwPopped_Style;
	LONG           dwPopped_ExStyle;
	int            nWidth;
	int            nHeight;
	HWND           hWndParent;
	StateVarType  *pStateVar;
	int            nX0;
	int            nY0;
	COLORREF       dwON_TextColor;
	COLORREF       dwOFF_TextColor;
	COLORREF       dwON_Background;
	COLORREF       dwOFF_Background;
	COLORREF       dwDefaultBackground;
	bool           bButtonLike_PoppedBorder;
	bool           bButtonLike_PushedBorder;
	bool           bTextXCenter;
	
	StateFlashButtonInitData():
	nFontDY(17),
	nFontWeight(500),
	pON_Text(NULL),
	pOFF_Text(NULL),
	dwON_TextColor(-1),
	dwOFF_TextColor(-1),
	PushMode(PUSH_NONE),
	bInverted(false),
	nPushed_Text_X_Offset(0),
	nPushed_Text_Y_Offset(0),
	nPopped_Text_X_Offset(0),
	nPopped_Text_Y_Offset(0),
	nUserText_X_Offset(0),
	dwPushed_Style( WS_CHILD|WS_VISIBLE ),
	dwPushed_ExStyle(0),
	dwPopped_Style( WS_CHILD|WS_VISIBLE ),
	dwPopped_ExStyle(0),
	nWidth(10),
	nHeight(18),
	hWndParent(NULL),
	pStateVar(NULL),
	nX0(0),
	nY0(0),
	dwON_Background( DEF_BACKGROUND ),
	dwOFF_Background( DEF_BACKGROUND ),
	dwDefaultBackground( DEF_BACKGROUND ),
	bButtonLike_PoppedBorder(true),
	bButtonLike_PushedBorder(true),
	bTextXCenter(false)
	{}
};

class StateFlashButtonClass : public FlashButtonClass {
public:

	StateFlashButtonClass( StateFlashButtonInitData *pInitData ):
	FlashButtonClass( MakeFlashButtonInitData( pInitData ) ),
	pStateVar( pInitData->pStateVar ),
	bInternalSet(false)
	{
		char acControlName[1024];
		sprintf(acControlName,"%s_StateFlashButton_%i", acModuleName, ++nObjectsCount );
		
		sControlName = acControlName;

		if( pStateVar ) { pStateVar->SetCallback( sControlName.c_str(), fastdelegate::MakeDelegate(this, &StateFlashButtonClass::StateCallback) ); }
		else            { sUserText = " "; DisableButton(); }
	}


	virtual void Set_pStateVar( StateVarType *pStateVAR )
	{
		if( !pStateVar && pStateVAR )
		{
			pStateVar = pStateVAR;
			sUserText.clear();
			EnableButton();
			pStateVar->SetCallback( sControlName.c_str(), fastdelegate::MakeDelegate(this, &StateFlashButtonClass::StateCallback) );
		}
		else
		if( pStateVar && !pStateVAR )
		{
			pStateVar = pStateVAR;
			sUserText = " ";
			DisableButton();
			pStateVar->SetCallback( sControlName.c_str(), NULL );
		}
		else
		{
			pStateVar->SetCallback( sControlName.c_str(), NULL );
			pStateVar = pStateVAR;
			pStateVar->SetCallback( sControlName.c_str(), fastdelegate::MakeDelegate(this, &StateFlashButtonClass::StateCallback) );
		}
	}
//
protected:
	virtual ~StateFlashButtonClass(){}
	
	StateVarType       *pStateVar;
	
	FlashButtonInitData   InitData;
	FlashButtonInitData*  MakeFlashButtonInitData( StateFlashButtonInitData *pInitData )
	{
		InitData.nFontDY            = pInitData->nFontDY;
		InitData.nFontWeight        = pInitData->nFontWeight;
		InitData.pON_Text           = pInitData->pON_Text;
		InitData.pOFF_Text          = pInitData->pOFF_Text;
		InitData.PushMode           = pInitData->PushMode;
		InitData.bInverted          = pInitData->bInverted;
		InitData.nPushed_Text_X_Offset  = pInitData->nPushed_Text_X_Offset;
		InitData.nPushed_Text_Y_Offset  = pInitData->nPushed_Text_Y_Offset;
		InitData.nPopped_Text_X_Offset  = pInitData->nPopped_Text_X_Offset;
		InitData.nPopped_Text_Y_Offset  = pInitData->nPopped_Text_Y_Offset;
		InitData.nUserText_X_Offset     = pInitData->nUserText_X_Offset;
		InitData.dwPushed_Style         = pInitData->dwPushed_Style;
		InitData.dwPushed_ExStyle       = pInitData->dwPushed_ExStyle;
		InitData.dwPopped_Style         = pInitData->dwPopped_Style;
		InitData.dwPopped_ExStyle       = pInitData->dwPopped_ExStyle;
		InitData.nWidth             = pInitData->nWidth;
		InitData.nHeight            = pInitData->nHeight;
		InitData.hWndParent         = pInitData->hWndParent;
		InitData.hWndReceiver        = NULL;
		InitData.nX0                = pInitData->nX0;
		InitData.nY0                = pInitData->nY0;
		InitData.dwON_TextColor     = pInitData->dwON_TextColor;
		InitData.dwOFF_TextColor    = pInitData->dwOFF_TextColor;
		InitData.dwON_Background    = pInitData->dwON_Background;
		InitData.dwOFF_Background   = pInitData->dwOFF_Background;
		
		InitData.dwDefaultBackground = pInitData->dwDefaultBackground;
		
		InitData.bButtonLike_PoppedBorder = pInitData->bButtonLike_PoppedBorder;
		InitData.bButtonLike_PushedBorder  = pInitData->bButtonLike_PushedBorder;
		InitData.bTextXCenter           = pInitData->bTextXCenter;

		InitData.Push_Delegate = fastdelegate::MakeDelegate(this, &StateFlashButtonClass::PushController);
		
		return &InitData;
	}

	bool bInternalSet;

	void StateCallback( const char*, int nVal )
	{
		if( nVal == NOT_EXIST )
		{
			SetState( bInverted );
			pStateVar = NULL;
			sUserText = " ";
			DisableButton();
			return;
		}

		if( !bInternalSet )	{ SetState( nVal );	}
	}

	void PushController( int nState )
	{
		bInternalSet = true;
		if( pStateVar ) { pStateVar->Set( nState ); }
		bInternalSet = false;
	}

private:
	static unsigned int nObjectsCount;
};

////////////////////////////////////////////////////////////

struct PushButtonInitData {
	dword        dwLparam;
	int          nFontDY;
	int          nFontWeight;
	char        *pON_Text;
	char        *pOFF_Text;
	PushModeType PushMode;
	bool         bInverted;
	int          nWidth;
	int          nHeight;
	HWND         hWndParent;
	HWND         hWndReceiver;
	int          nX0;
	int          nY0;
	bool         bTextXCenter;
	
	fastdelegate::FastDelegate1<int> Push_Delegate;
	
	PushButtonInitData():
	dwLparam(0),
	nFontDY(17),
	nFontWeight(500),
	pON_Text(NULL),
	pOFF_Text(NULL),
	PushMode(PUSH_NONE),
	bInverted(false),
	nWidth(10),
	nHeight(18),
	hWndParent(NULL),
	hWndReceiver(NULL),
	nX0(0),
	nY0(0),
	bTextXCenter(true)
	{}
};

class PushButtonClass : public ControlsClass {
public:
	static const char* const CLASS_NAME; // = "PushButton";
	
	PushButtonClass( PushButtonInitData* );

	virtual int  Width() { return nWidth; }
	virtual int  Height(){ return nHeight; }
	virtual void SetX0( int nX0 );
	virtual void SetY0( int Y0 )
	{
		nY0 = Y0;
		MoveWindow( hWndPush, nX0, nY0, nWidth, nHeight, true );
	}
	virtual void Show(){ ShowWindow( hWndPush, SW_SHOW ); }
	virtual void Hide(){ ShowWindow( hWndPush, SW_HIDE ); }
	virtual void RePaint()
	{
		if( bPushState != bInverted )
		{ 
			SendMessage( hWndPush, WM_SETTEXT, 0, (LPARAM)sON_Text.c_str() );
		}
		else
		{
			SendMessage( hWndPush, WM_SETTEXT, 0, (LPARAM)sOFF_Text.c_str() );
		}
	}
	
	virtual void SetState( int nState, bool bTemporal = false );
	virtual void SetON_Text( const char *pcText )
	{
		sON_Text = pcText;
		if( bPushState != bInverted )
		{ 
			SendMessage( hWndPush, WM_SETTEXT, 0, (LPARAM)sON_Text.c_str() );
		}
	}
	virtual void SetOFF_Text( const char *pcText )
	{
		sOFF_Text = pcText;
		if( bPushState == bInverted )
		{ 
			SendMessage( hWndPush, WM_SETTEXT, 0, (LPARAM)sOFF_Text.c_str() );
		}
	}
	
	virtual void DisableButton();
	virtual void EnableButton();
	virtual int  GetIdealWidth();	

protected:
	virtual ~PushButtonClass();

	static std::map<HWND,PushButtonClass*>  mpThis;
	static WNDPROC                          wpOrigButtonProc;
	static HWND                             hWndButton_Last;
	static PushButtonClass                 *pThis;

	static LRESULT NewButtonProc( HWND hWndButton, UINT uMsg, WPARAM wParam, LPARAM lParam );
	
	void  InitInternals( PushButtonInitData *pInitData );
	
	bool  bPushState;
	fastdelegate::FastDelegate1<int> Push_Delegate;

	HWND  hWndPush;
	HDC   hDCPush;
	
	HWND  hWndParent;
	HWND  hWndReceiver;

	dword        dwLparam;
	int          nStyle;
	int          nX0;
	int          nY0;
	int          nWidth;
	int          nHeight;
	PushModeType PushMode;
	bool         bInverted;
	
	
	HFONT fontX;
	int   nFontDY;
	int   nFontDX;

	HGDIOBJ  hBackground;
	
	std::string sON_Text;
	std::string sOFF_Text;
	
	bool   bDisabled;
	bool   bOnColorsEnabled;
	
	bool   bObjectDestroing;
	bool   bWindowDestroed;
};

////////////////////////////////////////////////////////////

struct StatePushButtonInitData {
	int            nFontDY;
	int            nFontWeight;
	char          *pON_Text;
	char          *pOFF_Text;
	PushModeType   PushMode;
	bool           bInverted;
	int            nWidth;
	int            nHeight;
	HWND           hWndParent;
	StateVarType  *pStateVar;
	int            nX0;
	int            nY0;
	bool           bTextXCenter;
	
	StatePushButtonInitData():
	nFontDY(17),
	nFontWeight(500),
	pON_Text(NULL),
	pOFF_Text(NULL),
	PushMode(PUSH_NONE),
	bInverted(false),
	nWidth(10),
	nHeight(18),
	hWndParent(NULL),
	pStateVar(NULL),
	nX0(0),
	nY0(0),
	bTextXCenter(true)
	{}
};

class StatePushButtonClass : public PushButtonClass {
public:

	StatePushButtonClass( StatePushButtonInitData *pInitData ):
	PushButtonClass( MakePushButtonInitData( pInitData ) ),
	nStatesSum(0),
	bInternalSet(false)
	{
		char acControlName[1024];
		sprintf(acControlName,"%s_StatePushButton_%i", acModuleName, ++nObjectsCount );
		
		sControlName = acControlName;
		
		StateVarType *pStateVar = pInitData->pStateVar;
		
		if( pStateVar ) 
		{
			vpStateVars.push_back( pStateVar );
			pStateVar->SetCallback( sControlName.c_str(), fastdelegate::MakeDelegate(this, &StatePushButtonClass::StateCallback) );
		}
		else
		{
			SendMessage( hWndPush, WM_SETTEXT, 0, (LPARAM)"" );
			DisableButton();
		}
	}

	virtual bool IsThemed(){ return IsAppThemed(); }
	
	virtual void Add_pStateVar( StateVarType *pStateVar )
	{
		if( !pStateVar ) { return; }
		
		if( !vpStateVars.size() )
		{
			EnableButton();
			RePaint();
		}

		vpStateVars.push_back( pStateVar );
		pStateVar->SetCallback( sControlName.c_str(), fastdelegate::MakeDelegate(this, &StatePushButtonClass::StateCallback) );
	}
		
//
protected:
	virtual ~StatePushButtonClass(){}
	
	std::vector <StateVarType*>  vpStateVars;
	
	PushButtonInitData   InitData;
	PushButtonInitData*  MakePushButtonInitData( StatePushButtonInitData *pInitData )
	{
		InitData.nWidth       = pInitData->nWidth;
		InitData.nHeight      = pInitData->nHeight;
		InitData.hWndParent   = pInitData->hWndParent;
		InitData.hWndReceiver  = NULL;
		InitData.nX0          = pInitData->nX0;
		InitData.nY0          = pInitData->nY0;
		InitData.nFontDY      = pInitData->nFontDY;
		InitData.nFontWeight  = pInitData->nFontWeight;
		InitData.PushMode     = pInitData->PushMode;
		InitData.pOFF_Text    = pInitData->pOFF_Text;
		InitData.pON_Text     = pInitData->pON_Text;
		InitData.bInverted    = pInitData->bInverted;
		InitData.bTextXCenter = pInitData->bTextXCenter;

		InitData.Push_Delegate = fastdelegate::MakeDelegate(this, &StatePushButtonClass::PushController);

		return &InitData;
	}

	bool bInternalSet;

	void StateCallback( const char*, int nVal )
	{
		if( bInternalSet || nVal < 0 )	{ return; }

		if( nVal ) { nStatesSum++; }
		else       { nStatesSum--; }
		
		int nStates = vpStateVars.size();
		
		if( nStatesSum >= nStates )
		{
			nStatesSum =  nStates;
			SetState( ON );
		}
		else
		if( nStatesSum <= 0 ) 
		{
			nStatesSum = 0;
			SetState( OFF );
		}
	}

	void PushController( int nState )
	{
		int nStates = vpStateVars.size();

		bInternalSet = true;
		for( int i = 0 ; i < nStates ; i++ )
		{
			vpStateVars[i]->Set( nState );
		}
		bInternalSet = false;
		
		if( nState ) { nStatesSum = nStates; }
		else         { nStatesSum = 0;       }
	}
	
private:
	int	nStatesSum;

	static unsigned int nObjectsCount;
};

////////////////////////////////////////////////////////////

struct PushMenuInitData {
	dword        dwLparam;
	int          nButtonPixelWidth;
	int          nButtonPixelHeight;
	HWND         hWndParent;
	HWND         hWndReceiver;
	HWND         hWndMain;
	int          nX0;
	int          nY0;
	COLORREF     dwListBackground;
	COLORREF     dwActiveBackground;
	
	fastdelegate::FastDelegate1<int> ItemSelected_Delegate;
	
	PushMenuInitData():
	dwLparam(0),
	nButtonPixelWidth(10),
	nButtonPixelHeight(18),
	hWndParent(NULL),
	hWndReceiver(NULL),
	hWndMain(NULL),
	nX0(0),
	nY0(0),
	dwListBackground(0),
	dwActiveBackground(0)
	{}
};


struct PushMenuItemType {
	std::string  sName;
	int          nVal;
	int          nListDX;
};


#define PUSH_MENU_ITEM_SELECTED   42000

class PushMenuClass : public ControlsClass {
public:
	static const char* const CLASS_NAME; // = "PushMenu";
	
	PushMenuClass( PushMenuInitData* );

	virtual int  Width() { return nButtonPixelWidth; }
	virtual int  Height(){ return nButtonPixelHeight; }
	virtual void Show(){ ShowWindow( hWndPush, SW_SHOW ); }
	virtual void Hide(){ ShowWindow( hWndPush, SW_HIDE ); }
	virtual void SetY0( int Y0 )
	{
		nY0 = Y0;
		MoveWindow( hWndPush, nX0, nY0, nButtonPixelWidth, nButtonPixelHeight, true );
	}
	virtual void SetX0( int X0 )
	{
		nX0 = X0;
		MoveWindow( hWndPush, nX0, nY0, nButtonPixelWidth, nButtonPixelHeight, true );
	}
	virtual void DisableButton()
	{ 
		bDisabled = true;
		EnableWindow( hWndPush, FALSE );
	}

	virtual void EnableButton()
	{ 
		bDisabled = false;
		EnableWindow( hWndPush, TRUE );
	}
	//
	void Clear();
	void AddItem( const char *acItemName, int nItemVal, int nListDX );
	void SetCurrentItemByVal( int nItemVal );
	
protected:
	virtual ~PushMenuClass();

	HDC   hDCList;

	static std::map<HWND,PushMenuClass*>  mpThis;
	static WNDPROC                        wpOrigButtonProc;
	static HWND                           hWndButton_Last;
	static PushMenuClass                 *pThis;

	static LRESULT NewButtonProc( HWND hWndButton, UINT uMsg, WPARAM wParam, LPARAM lParam );
	
	void  InitInternals( PushMenuInitData *pInitData );
	
	std::vector<PushMenuItemType> vItems;
	
	HGDIOBJ  hListBack;
	HGDIOBJ  hListActiveBack;
	
	void ShowList();

	virtual void RePaint(){}

	static LRESULT CALLBACK WndProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam );
	LRESULT CALLBACK ListWndProc( HWND, UINT, WPARAM, LPARAM );

	bool  bPushState;
	HWND  hWndOldFocus;

	HWND  hWndList;
	HWND  hWndPush;
	
	HWND  hWndMain;
	HWND  hWndParent;
	HWND  hWndReceiver;
	
	dword        dwLparam;

	COLORREF     dwListBackground;
	COLORREF     dwActiveBackground;

	fastdelegate::FastDelegate1<int> ItemSelected_Delegate;
	
	int   nCurrentItem;
	int   nCurrItem;
	int   nPrevItem;

	int   nList_X0;
	int   nList_Y0;
	int   nList_dX;
	int   nList_dY;

	int   nMenuCharsWidth;
	int   nButtonPixelWidth;
	int   nButtonPixelHeight;
	
	
	bool  bKillFocus;

	HFONT fontPush;
	HFONT fontList;
	
	int   nFontDY;
	int   nFontDX;
	
	bool bObjectDestroing;
	bool bWindowDestroed;
	
	int  nX0;
	int  nY0;
	
	bool bDisabled;
};


////////////////////////////////////////////////////////////


struct IntegerParameterPushMenuInitData {
	int                nButtonPixelWidth;
	int                nButtonPixelHeight;
	HWND               hWndParent;
	HWND               hWndMain;
	IntegerParameter  *pParamVar;
	int                nX0;
	int                nY0;
	COLORREF           dwListBackground;
	COLORREF           dwActiveBackground;
	
	IntegerParameterPushMenuInitData():
	nButtonPixelWidth(10),
	nButtonPixelHeight(18),
	hWndParent(NULL),
	hWndMain(NULL),
	pParamVar(NULL),
	nX0(0),
	nY0(0),
	dwListBackground(0),
	dwActiveBackground(0)
	{}
};


class IntegerParameterPushMenuClass : public PushMenuClass {
public:
	IntegerParameterPushMenuClass( IntegerParameterPushMenuInitData *pInitData ):
	PushMenuClass( MakePushMenuInitData(pInitData) ),
	pParamVar( pInitData->pParamVar ),
	bInternalSet(false)
	{
		char acControlName[1024];
		sprintf(acControlName,"%s_IntegerParameterPushMenu_%i", acModuleName, ++nObjectsCount );
		
		sControlName = acControlName;
		
		if( pParamVar ) { pParamVar->SetCallback( sControlName.c_str(), fastdelegate::MakeDelegate(this, &IntegerParameterPushMenuClass::ParameterCallback) ); }
		else		    { DisableButton(); }
	}

	virtual bool IsThemed(){ return IsAppThemed(); }

	virtual void Set_pParamVar( IntegerParameter  *pParamVAR )
	{
		if( !pParamVar && pParamVAR )
		{
			pParamVar = pParamVAR;
			EnableButton();
			pParamVar->SetCallback( sControlName.c_str(), fastdelegate::MakeDelegate(this, &IntegerParameterPushMenuClass::ParameterCallback) );
		}
		else
		if( pParamVar && !pParamVAR )
		{
			pParamVar = pParamVAR;
			SendMessage( hWndPush, WM_SETTEXT, 0, (LPARAM)"" ); 
			DisableButton();
			pParamVar->SetCallback( sControlName.c_str(), NULL );
		}
		else
		{
			pParamVar->SetCallback( sControlName.c_str(), NULL );
			pParamVar = pParamVAR;
			pParamVar->SetCallback( sControlName.c_str(), fastdelegate::MakeDelegate(this, &IntegerParameterPushMenuClass::ParameterCallback) );
		}
	}
	
//
protected:
	virtual ~IntegerParameterPushMenuClass(){}
	
	IntegerParameter           *pParamVar;
	
	PushMenuInitData   InitData;
	PushMenuInitData*  MakePushMenuInitData( IntegerParameterPushMenuInitData *pInitData )
	{
		InitData.nButtonPixelWidth  = pInitData->nButtonPixelWidth;
		InitData.nButtonPixelHeight = pInitData->nButtonPixelHeight;
		InitData.hWndParent         = pInitData->hWndParent;
		InitData.hWndReceiver        = NULL;
		InitData.hWndMain           = pInitData->hWndMain;
		InitData.nX0                = pInitData->nX0;
		InitData.nY0                = pInitData->nY0;
		InitData.dwListBackground   = pInitData->dwListBackground;
		InitData.dwActiveBackground = pInitData->dwActiveBackground;

		InitData.ItemSelected_Delegate = fastdelegate::MakeDelegate(this, &IntegerParameterPushMenuClass::PusMenuItemSelected_Callback);
		
		return &InitData;
	}

	bool bInternalSet;

	void ParameterCallback( const char*, int nVal )
	{
		if( !bInternalSet )	{ SetCurrentItemByVal( nVal ); }
	}

	void PusMenuItemSelected_Callback( int nItem )
	{
		bInternalSet = true;
		if( pParamVar ) { pParamVar->Set( nItem ); }
		bInternalSet = false;
	}
//
private:
	static unsigned int nObjectsCount;
};

////////////////////////////////////////////////////////////

struct PushMenu2InitData {
	const char  *acText;
	dword        dwLparam;
	int          nButtonPixelWidth;
	int          nButtonPixelHeight;
	HWND         hWndParent;
	HWND         hWndReceiver;
	HWND         hWndMain;
	int          nX0;
	int          nY0;
	COLORREF     dwListBackground;
	COLORREF     dwActiveBackground;
	
	fastdelegate::FastDelegate2<const char*,int> ItemSelected_Delegate;
	
	PushMenu2InitData():
	acText(NULL),
	dwLparam(0),
	nButtonPixelWidth(10),
	nButtonPixelHeight(18),
	hWndParent(NULL),
	hWndReceiver(NULL),
	hWndMain(NULL),
	nX0(0),
	nY0(0),
	dwListBackground(0),
	dwActiveBackground(0)
	{}
};


struct PushMenu2ItemType {
	std::string  sName;
	int          nVal;
	int          nListDX;
};

#define PUSH_MENU_2_ITEM_SELECTED   42100

class PushMenu2Class : public ControlsClass {
public:
	static const char* const CLASS_NAME; // = "PushMenu2";
	
	PushMenu2Class( PushMenu2InitData* );
	
	virtual int  Width() { return nButtonPixelWidth; }
	virtual int  Height(){ return nButtonPixelHeight; }
	virtual void SetY0( int Y0 )
	{
		nY0 = Y0;
		MoveWindow( hWndPush, nX0, nY0, nButtonPixelWidth, nButtonPixelHeight, true );
	}
	virtual void SetX0( int X0 )
	{
		nX0 = X0;
		MoveWindow( hWndPush, nX0, nY0, nButtonPixelWidth, nButtonPixelHeight, true );
	}
	virtual void Show(){ ShowWindow( hWndPush, SW_SHOW ); }
	virtual void Hide(){ ShowWindow( hWndPush, SW_HIDE ); }

	void AddItem( const char *acItemName, int nItemVal, int nListDX );
	void SetCurrentItemByName( const char *pcName );
	void SetText( const char *pcText )
	{
		SetWindowText( hWndPush, pcText );
	}
//	
protected:
	virtual ~PushMenu2Class();

	static std::map<HWND,PushMenu2Class*>  mpThis;
	static WNDPROC                         wpOrigButtonProc;
	static HWND                            hWndButton_Last;
	static PushMenu2Class                 *pThis;

	static LRESULT NewButtonProc( HWND hWndButton, UINT uMsg, WPARAM wParam, LPARAM lParam );

	HDC   hDCList;
	
	dword dwLparam;
	
	void  InitInternals( PushMenu2InitData *pInitData );
	
	std::vector<PushMenu2ItemType> vItems;
	
	HGDIOBJ  hListBack;
	HGDIOBJ  hListActiveBack;
	
	void ShowList();
	
	virtual void RePaint(){}
	
	std::string sText;

	static LRESULT CALLBACK WndProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam );
	LRESULT CALLBACK ListWndProc( HWND, UINT, WPARAM, LPARAM );

	bool  bPushState;
	HWND  hWndOldFocus;

	HWND  hWndList;
	HWND  hWndPush;
	
	HWND  hWndMain;
	HWND  hWndParent;
	HWND  hWndReceiver;

	COLORREF     dwListBackground;
	COLORREF     dwActiveBackground;
	
	fastdelegate::FastDelegate2<const char*,int> ItemSelected_Delegate;
	
	int   nCurrentItem;
	int   nCurrItem;
	int   nPrevItem;

	int   nList_X0;
	int   nList_Y0;
	int   nList_dX;
	int   nList_dY;

	int   nButtonPixelWidth;
	int   nButtonPixelHeight;
	
	
	bool  bKillFocus;

	HFONT fontPush;
	HFONT fontList;

	int   nFontDY;
	int   nFontDX;

	bool bObjectDestroing;
	bool bWindowDestroed;

	int  nX0;
	int  nY0;
};


////////////////////////////////////////////////////////////


struct StringParameterPushMenu2InitData {
	const char        *acText;
	int               nButtonPixelWidth;
	int               nButtonPixelHeight;
	HWND              hWndParent;
	HWND              hWndMain;
	StringParameter  *pParamVar;
	int               nX0;
	int               nY0;
	COLORREF          dwListBackground;
	COLORREF          dwActiveBackground;
	
	StringParameterPushMenu2InitData():
	acText(NULL),
	nButtonPixelWidth(10),
	nButtonPixelHeight(18),
	hWndParent(NULL),
	hWndMain(NULL),
	pParamVar(NULL),
	nX0(0),
	nY0(0),
	dwListBackground(0),
	dwActiveBackground(0)
	{}
};


class StringParameterPushMenu2Class : public PushMenu2Class {
public:
	StringParameterPushMenu2Class( StringParameterPushMenu2InitData *pInitData ):
	PushMenu2Class( MakePushMenu2InitData(pInitData) ),
	ParamVar( *(pInitData->pParamVar) ),
	bInternalSet(false)
	{
		ParamVar.SetCallback( "StringParameterPushMenu2Class", fastdelegate::MakeDelegate(this, &StringParameterPushMenu2Class::ParameterCallback), false );
	}
	virtual bool IsThemed(){ return IsAppThemed(); }

protected:
	virtual ~StringParameterPushMenu2Class(){}
	
	StringParameter&  ParamVar;
	
	PushMenu2InitData   InitData;
	PushMenu2InitData*  MakePushMenu2InitData( StringParameterPushMenu2InitData *pInitData )
	{
		InitData.acText             = pInitData->acText;
		InitData.nButtonPixelWidth  = pInitData->nButtonPixelWidth;
		InitData.nButtonPixelHeight = pInitData->nButtonPixelHeight;
		InitData.hWndParent         = pInitData->hWndParent;
		InitData.hWndReceiver        = NULL;
		InitData.hWndMain           = pInitData->hWndMain;
		InitData.nX0                = pInitData->nX0;
		InitData.nY0                = pInitData->nY0;
		InitData.dwListBackground   = pInitData->dwListBackground;
		InitData.dwActiveBackground = pInitData->dwActiveBackground;

		InitData.ItemSelected_Delegate = fastdelegate::MakeDelegate(this, &StringParameterPushMenu2Class::PusMenuItemSelected_Callback);

		return &InitData;
	}

	bool bInternalSet;

	void ParameterCallback( const char*, const char *acItem )
	{
		if( !bInternalSet )	{ SetCurrentItemByName( acItem ); }
	}

	void PusMenuItemSelected_Callback( const char *acItem, int nVal )
	{
		bInternalSet = true;
		ParamVar.Set( acItem );
		bInternalSet = false;
	}
};

////////////////////////////////////////////////////////////

struct PushMenu3InitData {
	const char  *acText;
	dword        dwLparam;
	int          nButtonPixelWidth;
	int          nButtonPixelHeight;
	HWND         hWndParent;
	HWND         hWndMain;
	int          nX0;
	int          nY0;
	COLORREF     dwListBackground;
	COLORREF     dwActiveBackground;
	
	fastdelegate::FastDelegate3< const char*, const char*, dword > ItemSelected_Delegate;
	
	PushMenu3InitData():
	acText(NULL),
	dwLparam(0),
	nButtonPixelWidth(10),
	nButtonPixelHeight(18),
	hWndParent(NULL),
	hWndMain(NULL),
	nX0(0),
	nY0(0),
	dwListBackground(0),
	dwActiveBackground(0)
	{}
};


struct PushMenu3ItemType {
	std::string  sName;
	std::string  sVal;
	dword        dwData;
	int          nListDX;
};

class PushMenu3Class : public ControlsClass {
public:
	static const char* const CLASS_NAME; // = "PushMenu3";
	
	PushMenu3Class( PushMenu3InitData* );
	
	virtual int  Width() { return nButtonPixelWidth; }
	virtual int  Height(){ return nButtonPixelHeight; }
	virtual void SetY0( int Y0 )
	{
		nY0 = Y0;
		MoveWindow( hWndPush, nX0, nY0, nButtonPixelWidth, nButtonPixelHeight, true );
	}
	virtual void SetX0( int X0 )
	{
		nX0 = X0;
		MoveWindow( hWndPush, nX0, nY0, nButtonPixelWidth, nButtonPixelHeight, true );
	}
	virtual void Show(){ ShowWindow( hWndPush, SW_SHOW ); }
	virtual void Hide(){ ShowWindow( hWndPush, SW_HIDE ); }

	void AddItem( const char *acItemName, const char *acItemVal, dword dwItemData, int nListDX = 0 );
	void ClearItems();
	void SetCurrentItemByData( dword dwData );
	void SetText( const char *pcText )
	{
		SetWindowText( hWndPush, pcText );
	}
//	
protected:
	virtual ~PushMenu3Class();

	static std::map<HWND,PushMenu3Class*>  mpThis;
	static WNDPROC                         wpOrigButtonProc;
	static HWND                            hWndButton_Last;
	static PushMenu3Class                 *pThis;

	static LRESULT NewButtonProc( HWND hWndButton, UINT uMsg, WPARAM wParam, LPARAM lParam );

	HDC   hDCList;
	
	dword dwLparam;
	
	void  InitInternals( PushMenu3InitData *pInitData );
	
	std::vector<PushMenu3ItemType> vItems;
	
	HGDIOBJ  hListBack;
	HGDIOBJ  hListActiveBack;
	
	void ShowList();
	
	virtual void RePaint(){}
	
	std::string sText;

	static LRESULT CALLBACK WndProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam );
	LRESULT CALLBACK ListWndProc( HWND, UINT, WPARAM, LPARAM );

	bool  bPushState;
	HWND  hWndOldFocus;

	HWND  hWndList;
	HWND  hWndPush;
	
	HWND  hWndMain;
	HWND  hWndParent;

	COLORREF     dwListBackground;
	COLORREF     dwActiveBackground;
	
	fastdelegate::FastDelegate3< const char*, const char*, dword > ItemSelected_Delegate;
	
	int   nCurrentItem;
	int   nCurrItem;
	int   nPrevItem;

	int   nList_X0;
	int   nList_Y0;
	int   nList_dX;
	int   nList_dY;

	int   nButtonPixelWidth;
	int   nButtonPixelHeight;
	
	
	bool  bKillFocus;

	HFONT fontPush;
	HFONT fontList;

	int   nFontDY;
	int   nFontDX;

	bool bObjectDestroing;
	bool bWindowDestroed;

	int  nX0;
	int  nY0;
};

///////////////////////////////////////////////////////////////////

struct DriveButtonInitData {
	int          nFontDY;
	int          nFontWeight;
	int          nWidth;
	int          nHeight;
	HWND         hWndMain;
	HWND         hWndParent;
	HWND         hWndReceiver;
	int          nX0;
	int          nY0;
	
	fastdelegate::FastDelegate1<int> Push_Delegate;
	
	DriveButtonInitData():
	nFontDY(12),
	nFontWeight(600),
	nWidth(36),
	nHeight(18),
	hWndMain(NULL),
	hWndParent(NULL),
	hWndReceiver(NULL),
	nX0(0),
	nY0(-3)
	{}
};

struct DriveButtonListItemType {
	std::string  sVal;
};


class DriveButtonClass : public ControlsClass {
public:
	static const char* const CLASS_NAME; // = "DriveButton";
	
	DriveButtonClass( DriveButtonInitData* );

	virtual int  Width() { return nWidth; }
	virtual int  Height(){ return nHeight; }
	virtual void SetX0( int nNewX0 )
	{
		nX0 = nNewX0;
		MoveWindow( hWndPush, nX0, nY0, nWidth, nHeight, true ); 
	}
	virtual void SetY0( int nNewY0 )
	{
		nY0 = nNewY0;
		MoveWindow( hWndPush, nX0, nY0, nWidth, nHeight, true );
	}
	virtual void Show(){ ShowWindow( hWndPush, SW_SHOW ); bHidden = false; }
	virtual void Hide(){ ShowWindow( hWndPush, SW_HIDE ); bHidden = true;  }
	virtual void RePaint(){}
	
	virtual void SetDriveName( const char *acDriveName ) { sDriveName = acDriveName; Paint(); }
	virtual void SetMediaPath( const char *acMediaPath ) { sMediaPath = acMediaPath; Paint(); }
	
	virtual void Set_bMotorON( bool bMotor_ON ) 
	{	
		if( bMotorON != bMotor_ON )
		{
			bMotorON = bMotor_ON;
			SetWindowPos( hWndPush, NULL, 0,0,0,0, SWP_FRAMECHANGED|SWP_NOMOVE|SWP_NOSIZE|SWP_NOZORDER|SWP_NOCOPYBITS|SWP_SHOWWINDOW );
			Paint();
		}
	}
	virtual void Set_bProtected( bool bPROTECTED )
	{
		if( bPROTECTED != bProtected )
		{
			bProtected = bPROTECTED;
			SetWindowPos( hWndPush, NULL, 0,0,0,0, SWP_FRAMECHANGED|SWP_NOMOVE|SWP_NOSIZE|SWP_NOZORDER|SWP_NOCOPYBITS|SWP_SHOWWINDOW );
		}
	}

	virtual void Set_bSelected( bool bSELECTED )
	{
		if( bSELECTED != bSelected )
		{
			bSelected = bSELECTED;
			Paint();
		}
	}

	virtual void Set_nTrack( int nTRACK )
	{
		dword uTC = GetTickCount();
		if( nTRACK != nTrack )
		{
			nTrack = nTRACK;
			if( uTC - dwLastTrackChangeTC > 1000 ) 
			{
				dwLastTrackChangeTC = uTC;
				bFontBold = !bFontBold;
				if( bFontBold ) { SelectObject( hDCPush, fontPush1 ); }
				else            { SelectObject( hDCPush, fontPush2 ); }
				Paint();
			}
		}
		else
		if( !bFontBold && uTC - dwLastTrackChangeTC > 1000 )
		{
			bFontBold = true;
			dwLastTrackChangeTC = uTC;
		    SelectObject( hDCPush, fontPush1 );
			Paint();
		}
	}
	
protected:
	virtual ~DriveButtonClass();

	static LRESULT CALLBACK WndProc0( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam );
	LRESULT CALLBACK WndProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam );

	static LRESULT CALLBACK ListWndProc0( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam );
	LRESULT CALLBACK ListWndProc( HWND, UINT, WPARAM, LPARAM );
	
	HWND  hWndList;
	HDC   hDCList;
	
	void  InitInternals( DriveButtonInitData *pInitData );
	
	void  Paint();
	void  ShowList();

	int   nPushState;
	fastdelegate::FastDelegate1<int> Push_Delegate;

	std::string  sMediaPath;

	HWND  hWndPush;
	HDC   hDCPush;
	
	HWND  hWndParent;
	HWND  hWndMain;
	HWND  hWndReceiver;
	
	HBRUSH  hListBack;
	HBRUSH  hListActiveBack;

	int   nX0;
	int   nY0;
	int   nWidth;
	int   nHeight;
	
	bool  bFontBold;
	
	HFONT fontPush1;
	HFONT fontPush2;
	HFONT fontList;

	int   nFontDY;
	int   nFontDX;

	int   nCurrItem;
	int   nPrevItem;

	int   nList_X0;
	int   nList_Y0;
	int   nList_dX;
	int   nList_dY;

	HGDIOBJ  hBackground;
	
	std::string sDriveName;
	
	bool  bCtrlPressed;
	
	dword  dwWindowThreadID;
	
	bool   bDisabled;
	bool   bHidden;
	
	bool   bMotorON;
	bool   bProtected;
	bool   bSelected;
	int    nTrack;
	dword  dwLastTrackChangeTC;
	
	bool   bObjectDestroing;
	bool   bWindowDestroed;
};


#endif

