//
//  sysmac.h
//
#define EXPORT extern "C" __declspec(dllexport)

#include "err_log.h"

#define LogLastError() ErrLogLastErr( __FILE__, __LINE__ )
#define ErrLog(x)    ErrLogStrOut( (x), 'W', __FILE__, __LINE__ )
#define ErrLog_I(x)  ErrLogStrOut( (x), 'I', __FILE__, __LINE__ )
#define ErrLog_W(x)  ErrLogStrOut( (x), 'W', __FILE__, __LINE__ )
#define ErrLog_E(x)  ErrLogStrOut( (x), 'E', __FILE__, __LINE__ )
#define ErrLog_F(x)  ErrLogStrOut( (x), 'F', __FILE__, __LINE__ )


#if defined _M_IX86
#pragma comment(linker, "/manifestdependency:\"type='win32' name='Microsoft.Windows.Common-Controls' version='6.0.0.0' processorArchitecture='x86' publicKeyToken='6595b64144ccf1df' language='*'\"")
#elif defined _M_IA64
#pragma comment(linker, "/manifestdependency:\"type='win32' name='Microsoft.Windows.Common-Controls' version='6.0.0.0' processorArchitecture='ia64' publicKeyToken='6595b64144ccf1df' language='*'\"")
#elif defined _M_X64
#pragma comment(linker, "/manifestdependency:\"type='win32' name='Microsoft.Windows.Common-Controls' version='6.0.0.0' processorArchitecture='amd64' publicKeyToken='6595b64144ccf1df' language='*'\"")
#else
#pragma comment(linker, "/manifestdependency:\"type='win32' name='Microsoft.Windows.Common-Controls' version='6.0.0.0' processorArchitecture='*' publicKeyToken='6595b64144ccf1df' language='*'\"")
#endif

#pragma comment(lib, "comctl32.lib")

#ifndef SYSMAC_DEFS
#define SYSMAC_DEFS

#include <windows.h>
#include <string>
#include <vector>
#include <map>
#include <set>
#include <deque>
#include <algorithm>
#include <functional>

#include "err_log.h"

enum { LOG_OFF, LOG_ON, LOG_SUSPENDED };

enum NumberType { DECIM, SIGNED, OCTAL, HEX, BINARY, RGB };

//////////////////////////////////
typedef unsigned char        byte;
typedef unsigned short       word;
typedef unsigned long       dword;
typedef unsigned long long  qword;
//////////////////////////////////



#define BIT_0             01
#define BIT_1             02
#define BIT_2             04
#define BIT_3            010
#define BIT_4            020
#define BIT_5            040
#define BIT_6           0100
#define BIT_7           0200
#define BIT_8           0400
#define BIT_9          01000
#define BIT_10         02000
#define BIT_11         04000
#define BIT_12        010000
#define BIT_13        020000
#define BIT_14        040000
#define BIT_15       0100000
#define BIT_16       0200000
#define BIT_17       0400000
#define BIT_18      01000000
#define BIT_19      02000000
#define BIT_20      04000000
#define BIT_21     010000000
#define BIT_22     020000000
#define BIT_23     040000000
#define BIT_24    0100000000
#define BIT_25    0200000000
#define BIT_26    0400000000
#define BIT_27   01000000000
#define BIT_28   02000000000
#define BIT_29   04000000000
#define BIT_30  010000000000
#define BIT_31  020000000000


////////////////////////////////////////

#define BYTE_ALIGN  __declspec(align(1))
#define WORD_ALIGN  __declspec(align(2))
#define DWORD_ALIGN __declspec(align(4))
#define QWORD_ALIGN __declspec(align(8))

#define LOCK_VAR_TYPE  DWORD_ALIGN  LONG volatile

#define LOCK(var,sleep) var.Lock(sleep)
#define UNLOCK(var)     var.UnLock()


class LockVarType {
public:
	LockVarType():dwLockVar(0){}
	inline dword IsLocked(){ return dwLockVar; }
	inline void  Lock( int nSleepMS = 0 )
	{
		while( InterlockedCompareExchange( &dwLockVar, 1, 0 ) ) { Sleep( nSleepMS ); }
	}
	inline void  UnLock() { dwLockVar = 0; }
	inline bool  TimedLock( dword uTimeOut_MS, int nSleepMS = 0 ) 
	{
		dword  uTC = GetTickCount();
		while( InterlockedCompareExchange( &dwLockVar, 1, 0 ) ) 
		{
			if( GetTickCount() - uTC >= uTimeOut_MS ) { return false; }
			Sleep( nSleepMS );
		}
		return true;
	}
	inline bool TryLock() { return !InterlockedCompareExchange( &dwLockVar, 1, 0 ); }

protected:
	LOCK_VAR_TYPE  dwLockVar;
	
};


class SharedLockVarType {
public:
	SharedLockVarType( LOCK_VAR_TYPE *pdwLockVAR ):pdwLockVar( pdwLockVAR ){}
	inline dword IsLocked(){ return *pdwLockVar; }
	inline void  Set_pdwLockVar( LOCK_VAR_TYPE *pdwLockVAR ){ pdwLockVar = pdwLockVAR; }
	inline void  Lock( int nSleepMS = 0 )
	{
		while( InterlockedCompareExchange( pdwLockVar, 1, 0 ) ) { Sleep( nSleepMS ); }
	}
	inline void  UnLock() { *pdwLockVar = 0; }
	inline bool  TimedLock( dword uTimeOut_MS, int nSleepMS = 0 ) 
	{
		dword  uTC = GetTickCount();
		while( InterlockedCompareExchange( pdwLockVar, 1, 0 ) ) 
		{
			if( GetTickCount() - uTC >= uTimeOut_MS ) { return false; }
			Sleep( nSleepMS );
		}
		return true;
	}
	inline bool TryLock() { return !InterlockedCompareExchange( pdwLockVar, 1, 0 ); }
	
protected:
	LOCK_VAR_TYPE  *pdwLockVar;
	
};

class LockerType {
public:
	LockerType( LockVarType *pLockVAR ):
	pLockVar( pLockVAR )
	{
		pLockVar->Lock();
	}
	~LockerType()
	{
		pLockVar->UnLock();
	}
protected:
	LockVarType *pLockVar;
};


class ThreadLockType {
public:
	ThreadLockType(): dwLockVar(0), uNestingLevel(0) {}
	inline dword IsLocked(){ return dwLockVar; }
	inline void  Lock( int nSleepMS = 0 )
	{
		dword dwThisThreadId = GetCurrentThreadId();
		while( true )
		{
			dword dwLockThreadId = InterlockedCompareExchange( &dwLockVar, dwThisThreadId, 0 );
			if(  !dwLockThreadId || dwLockThreadId == dwThisThreadId ) 
			{
				uNestingLevel++;
				return;
			}
			Sleep( nSleepMS );
		}
	}
	inline void  UnLock() 
	{
		if( --uNestingLevel ) { return; }
		dwLockVar = 0;
	}
	inline bool  TimedLock( dword uTimeOut_MS, int nSleepMS = 0 ) 
	{
		dword  dwThisThreadId = GetCurrentThreadId();
		dword  uTC = GetTickCount();
		while( true ) 
		{
			dword dwLockThreadId = InterlockedCompareExchange( &dwLockVar, dwThisThreadId, 0 );
			if(  !dwLockThreadId || dwLockThreadId == dwThisThreadId ) 
			{
				uNestingLevel++;
				return true;
			}
			if( GetTickCount() - uTC >= uTimeOut_MS ) { return false; }
			Sleep( nSleepMS );
		}
	}
	inline bool TryLock() 
	{
		dword dwThisThreadId = GetCurrentThreadId();
		dword dwLockThreadId = InterlockedCompareExchange( &dwLockVar, dwThisThreadId, 0 );
		if(  !dwLockThreadId || dwLockThreadId == dwThisThreadId ) 
		{
			uNestingLevel++;
			return true;
		}
		return false;
	}

protected:
	LOCK_VAR_TYPE  dwLockVar;
	dword          uNestingLevel;
	
};




///////////////////////////////////////////////////////////////////////////////////////////////
WINBASEAPI BOOL WINAPI TryEnterCriticalSection( __inout LPCRITICAL_SECTION lpCriticalSection );
///////////////////////////////////////////////////////////////////////////////////////////////

class CriticalSectionType {
public:
	CriticalSectionType() { InitializeCriticalSection( &cs ); }
	inline void Enter()	{ EnterCriticalSection( &cs ); }
	inline void Leave() { LeaveCriticalSection( &cs ); }
	inline bool TimedEnter( dword uTimeOut_MS, int nSleepMS = 0 ) 
	{
		dword  uTC = GetTickCount();
		while( !TryEnterCriticalSection( &cs ) ) 
		{
			if( GetTickCount() - uTC >= uTimeOut_MS ) { return false; }
			Sleep( nSleepMS );
		}
		return true;
	}
	
protected:
	CRITICAL_SECTION  cs;
};

/////////////////////////////////////////////////////////////////////////

template< typename Type > class safe_vector : public std::vector< Type > {
public:
	inline void lock( int nSleepMS = 0 ) { LockVar.Lock( nSleepMS ); }
	inline void unlock() { LockVar.UnLock(); }
protected:
	LockVarType  LockVar;
};

/////////////////////////////////////////////////////////////////////////

template< typename Type > class safe_deque : public std::deque< Type > {
public:
	inline void lock( int nSleepMS = 0 ) { LockVar.Lock( nSleepMS ); }
	inline void unlock() { LockVar.UnLock(); }
protected:
	LockVarType  LockVar;
};

/////////////////////////////////////////////////////////////////////////

class safe_string : public std::string {
public:
	inline void operator =( const std::string sNewVal ) { std::string::operator =( sNewVal ); }
	inline void lock( int nSleepMS = 0 ) { LockVar.Lock( nSleepMS ); }
	inline void unlock() { LockVar.UnLock(); }
protected:
	LockVarType  LockVar;
};

//////////////////////////////////////////////////////////

class NoCopy {
protected:
	~NoCopy(){}
	NoCopy(){}
private:
	NoCopy( const NoCopy& );
};

class NoCopy2 {
protected:
	~NoCopy2(){}
	NoCopy2(){}
private:
	NoCopy2( const NoCopy2& );
	NoCopy2& operator = ( const NoCopy2& );
};

//////////////////////////////////////////////////////////////////////////

typedef void (__cdecl* void_DllFun_int_int_int_int)( int, int, int, int );
typedef void (__cdecl* void_DllFun_int_int)( int, int );
typedef void (__cdecl* void_DllFun_DWORD)( dword );
typedef void (__cdecl* void_DllFun_int)( int );
typedef void (__cdecl* void_DllFun_void)( void );
typedef void (__cdecl* void_DllFun_pchar)( const char* );
typedef void (__cdecl* void_DllFun_char)( char );
typedef void (__cdecl* void_DllFun_HWND)( HWND );
typedef bool (__cdecl* bool_DllFun_void)( void );


#endif

