//
//  stub_architecture.h - header for stub_architecture.cpp
//
#ifndef stub_architectureDefs
#define stub_architectureDefs 1


#include "UtilsLib\FastDelegate.h"
#include "UtilsLib\main_classes.h"
#include "UtilsLib\abstract_model.h"
#include "Main_module.API\config.h"
#include "UtilsLib\FrameStream.h"
#include "UtilsLib\FrameMonitor.h"


#define stub_architecture "stub"

//
//
//
class    stub_DeviceInterface;
typedef  std::vector<byte>  PortsVectorType;

//
//
//


#define  stub_BusInterface_CLASS_NAME   "stub_BusInterface"

const dword  stub_BusInterface_Generation = 1;
const dword  stub_BusInterface_Version    = 1;


class stub_BusInterface : public InterfaceClass {
public:
	static const char* const CLASS_NAME; // "stub_BusInterface"

	virtual stub_DeviceInterface** Get_vpPortOwners()=0;

protected:
	virtual ~stub_BusInterface(){}
	stub_BusInterface( ObjectClass *pOwner ):InterfaceClass( pOwner, CLASS_NAME, stub_BusInterface_Generation, stub_BusInterface_Version ){}
};

////////////////////////////////////////////////

#define  stub_BusClass_CLASS_NAME    "stub_Bus"
#define  stub_BusClass_OBJECT_TYPE   stub_architecture


class stub_BusClass : protected AbstractBusClass, protected stub_BusInterface {
public:
	static const char* const CLASS_NAME;      // "stub_Bus";

	static const char* const OBJECT_TYPE;     // "stub"
	static ObjectClass*      ObjectConstructor( ConfigInterface*, const char *acAlias );
	static InternalsType     InternalsRoot;

	static InternalsPointersVectorType  vInternalsPointers;
	static InternalsPointersMapType     mInternalsPointers;

protected:
	virtual ~stub_BusClass() {}

	virtual	ObjectClass*   Get_pOwner()	{ return (ObjectClass*)this; }

private:
	stub_BusClass( ConfigInterface *pConfig, const char *acAlias );

	stub_DeviceInterface*   vpPortOwners[256];
	stub_DeviceInterface**  Get_vpPortOwners() { return vpPortOwners; }

	void LinkDone( SocketClass* pSocket, SocketClass* );
	
	static unsigned int nObjectsCount;
};

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

enum stub_CPU_command_codes  : byte { NOP, TEST_PORT, JUMP_NOT_READY, READ_PORT, WRITE_PORT, INC, JUMP };


#define  stub_CPU_Class_CLASS_NAME    "stub_CPU"
#define  stub_CPU_Class_OBJECT_TYPE   stub_CPU_Class_CLASS_NAME

class stub_CPU_Class : protected AbstractCPU_Class {
public:
	static const char* const CLASS_NAME;      // "stub_CPU"
	
	static const char* const OBJECT_TYPE;     // "stub_CPU"
	static ObjectClass*     ObjectConstructor( ConfigInterface*, const char *acAlias );
	static InternalsType    InternalsRoot;

	static InternalsPointersVectorType  vInternalsPointers;
	static InternalsPointersMapType     mInternalsPointers;

protected:
	virtual ~stub_CPU_Class() {}
	
	word  uProgramCounter;
	
	void  CPU_Steps( dword nSteps );

	void  Shorten_Remaining_BurstSteps_to( int nSteps ) {}

private:
	stub_CPU_Class( ConfigInterface *pConfig, const char *acAlias );

	stub_DeviceInterface**  pvpPortOwners;
	
	bool  bPortReadyFlag;
	byte  uAccumulator;

	void LinkDone( SocketClass* pSocket, SocketClass* );

	static unsigned int nObjectsCount;
};


///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

#define  stub_DeviceInterface_CLASS_NAME   "stub_DeviceInterface"

const dword  stub_DeviceInterface_Generation = 1;
const dword  stub_DeviceInterface_Version    = 1;


class stub_DeviceInterface : public InterfaceClass {
public:
	static const char* const CLASS_NAME; // "stub_DeviceInterface"

	virtual const PortsVectorType * const Get_vPorts() { return &vPorts; }
	virtual byte                          ReadPort( byte uPort ) { return 0; };
	virtual void                          WritePort( byte uPort, byte uByte ) {};
	virtual bool                          TestPort( byte uPort ) { return true; };

protected:
	virtual ~stub_DeviceInterface(){}
	stub_DeviceInterface( ObjectClass *pOwner ):InterfaceClass( pOwner, CLASS_NAME, stub_DeviceInterface_Generation, stub_DeviceInterface_Version ){}

	PortsVectorType  vPorts;
};


//////////////////////////////////////////////////

#define  stub_Keyboard_Class_CLASS_NAME    "stub_Keyboard"
#define  stub_Keyboard_Class_OBJECT_TYPE   stub_Keyboard_Class_CLASS_NAME

class stub_Keyboard_Class : protected AbstractDeviceClass, protected stub_DeviceInterface {
public:
	static const char* const CLASS_NAME;      // "stub_Keyboard"
	
	static const char* const OBJECT_TYPE;     // "stub_Keyboard"
	static ObjectClass*     ObjectConstructor( ConfigInterface*, const char *acAlias );
	static InternalsType    InternalsRoot;

	static InternalsPointersVectorType  vInternalsPointers;
	static InternalsPointersMapType     mInternalsPointers;

protected:
	virtual ~stub_Keyboard_Class() 
	{
		bExit = true;
	}
	
	virtual void  PrepareEmulation() 
	{
		pConfigI->SetInstantCallback( fastdelegate::MakeDelegate(this, &stub_Keyboard_Class::InstantCallback) );
	}

	virtual byte  ReadPort( byte uPort );
	virtual bool  TestPort( byte uPort )
	{
		if( uPort == 'A' ) { return sInputBuf.size(); }
		return false;
	};

private:
	stub_Keyboard_Class( ConfigInterface *pConfig, const char *acAlias );
	
	bool abKeysPrev[256];
	const byte *aKeys;
	
	safe_string sInputBuf;
	
	void InstantCallback();
	
	bool bExit;
	
	static unsigned int nObjectsCount;
};


//////////////////////////////////////////////////

#define  stub_Serial_Class_CLASS_NAME    "stub_Serial"
#define  stub_Serial_Class_OBJECT_TYPE   stub_Serial_Class_CLASS_NAME

class stub_Serial_Class : protected AbstractDeviceClass, protected stub_DeviceInterface {
public:
	static const char* const CLASS_NAME;      // "stub_Serial"
	
	static const char* const OBJECT_TYPE;     // "stub_Serial"
	static ObjectClass*     ObjectConstructor( ConfigInterface*, const char *acAlias );
	static InternalsType    InternalsRoot;

	static InternalsPointersVectorType  vInternalsPointers;
	static InternalsPointersMapType     mInternalsPointers;

protected:
	virtual ~stub_Serial_Class() {}
	
	virtual void  PrepareEmulation() {}

	Input_SocketClass   ByteInputSocket;
	Output_SocketClass  ByteOutputSocket;

	void ByteReceiver( PVOID pData, SocketClass *pSender )
	{
		sInputBuf.lock();
		/////////////////
		sInputBuf += (byte)pData;
		/////////////////
		sInputBuf.unlock();
	}
	virtual byte  ReadPort( byte uPort ) 
	{
		if( uPort != 'C' || !sInputBuf.size() ) { return 0; }
		sInputBuf.lock();
		/////////////////
		byte ch = sInputBuf.c_str()[0];
		std::string sTmp = &sInputBuf.c_str()[1];
		sInputBuf = sTmp;
		/////////////////
		sInputBuf.unlock();
		return ch;
	};
	virtual void  WritePort( byte uPort, byte uByte ) 
	{
		if( uPort == 'D' )
		{
			ByteOutputSocket.Send( (PVOID)uByte );
		}
	};
	virtual bool  TestPort( byte uPort )
	{
		if( uPort == 'C' ) { return sInputBuf.size(); }
		else
		if( uPort == 'D' ) { return true; }
		return false;
	};
//
private:
	stub_Serial_Class( ConfigInterface *pConfig, const char *acAlias );
	
	safe_string      sInputBuf;
	
	static unsigned int nObjectsCount;
};


//////////////////////////////////////////////////

#define  stub_Video_Class_CLASS_NAME    "stub_Video"
#define  stub_Video_Class_OBJECT_TYPE   stub_Video_Class_CLASS_NAME

class stub_Video_Class : protected AbstractDeviceClass, protected stub_DeviceInterface, protected FrameMonitorClass {
public:
	static const char* const CLASS_NAME;      // "stub_Video"
	
	static const char* const OBJECT_TYPE;     // "stub_Video"
	static ObjectClass*     ObjectConstructor( ConfigInterface*, const char *acAlias );
	static InternalsType    InternalsRoot;

	static InternalsPointersVectorType  vInternalsPointers;
	static InternalsPointersMapType     mInternalsPointers;

protected:
	virtual ~stub_Video_Class() {}
	
	virtual void  PrepareEmulation() {}
	virtual void  WF_ActiveStateChanged( bool bActive );
	virtual void  WF_Tick();
	virtual void  Tick() {}

	virtual void  WritePort( byte uPort, byte uByte );

private:
	stub_Video_Class( ConfigInterface *pConfig, const char *acAlias );
	
	BGR_Frame_Type  Frame;
	
	static unsigned int nObjectsCount;
};





#endif



